@@ -1,4 +1,5 @@
 library(mirai)
+
 nanotest <- function(x) invisible(x || stop())
 nanotestn <- function(x) invisible(is.null(x) || stop())
 nanotestz <- function(x) invisible(x == 0L || stop())
@@ -7,33 +8,19 @@ nanotestp <- function(x) invisible(is.character(capture.output(print(x))) || sto
 nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop())
 
-nanotest(is.list(daemons()))
-nanotestz(daemons()[["connections"]])
-nanotestz(daemons()[["daemons"]])
-n <- 3L
-m <- mirai({
-  Sys.sleep(0.1)
-  q <- m + n + 1L
-  q / m
-}, m = 2L, .args = list(n), .timeout = 1000L)
-b <- m$data %>>% rnorm %>>% as.character()
-nanotestp(b)
-nanotest(inherits(call_mirai(m), "mirai"))
-nanotest(m$data == 3L || is_error_value(m$data))
-nanotest(identical(call_mirai(m), m))
-nanotest(is_mirai(m))
-nanotest(length(b) == 3L || length(b$data) == 3L)
-nanotest(is.character(b) || is.character(b$data))
-Sys.sleep(2.2)
-if (Sys.getenv("NOT_CRAN") == "true") {
+for (i in 1L) {
+
+  nanotest(is.list(daemons()))
+  nanotestz(daemons()[["connections"]])
+  nanotestz(daemons()[["daemons"]])
   nanotesto(daemons(1L, dispatcher = FALSE))
-  me <- mirai(mirai(), .timeout = 1000L)
+  me <- mirai(mirai::mirai(), .timeout = 2000L)
   nanotest(is_mirai_error(call_mirai(me)$data) || is_error_value(me$data))
   nanotest(!is_mirai_interrupt(me$data))
   nanotest(is_error_value(me[["data"]]))
   nanotestp(me$data)
   df <- data.frame(a = 1, b = 2)
-  dm <- mirai(as.matrix(df), .args = list(df), .timeout = 1000L)
+  dm <- mirai(as.matrix(df), .args = list(df), .timeout = 2000L)
   nanotest(is_mirai(call_mirai(dm)))
   nanotest(!unresolved(dm))
   nanotest(is.matrix(dm$data) || is_error_value(dm$data))
@@ -43,8 +30,40 @@ if (Sys.getenv("NOT_CRAN") == "true") {
   nanotestz(daemons(0L))
   nanotestz(daemons()[["connections"]])
   nanotestz(daemons()[["daemons"]])
+  nanotesterr(daemons(url = "URL"), "Invalid argument")
+  nanotesterr(daemons(-1), "zero or greater")
+  nanotesterr(daemons(raw(0L)), "must be numeric")
+  nanotesterr(server("URL"), "Invalid argument")
+  nanotesterr(dispatcher(client = "URL"), "at least one")
+  nanotestz(daemons(0L))
+  nanotest(is_mirai_interrupt(r <- mirai:::mk_interrupt_error()))
+  nanotestp(r)
   Sys.sleep(1L)
-  nanotest(daemons(url = value <- sprintf(mirai:::.urlfmt, nanonext::random()), dispatcher = FALSE) == value)
+
+  Sys.info()[["sysname"]] != "Windows" || length(R.version$crt) || break
+  n <- 3L
+  m <- mirai({
+    Sys.sleep(0.1)
+    q <- m + n + 1L
+    q / m
+  }, m = 2L, .args = list(n), .timeout = 2000L)
+  b <- m$data %>>% rnorm %>>% as.character()
+  nanotestp(b)
+  nanotest(inherits(call_mirai(m), "mirai"))
+  nanotest(m$data == 3L || is_error_value(m$data))
+  nanotest(identical(call_mirai(m), m))
+  nanotest(is_mirai(m))
+  nanotest(length(b) == 3L || length(b$data) == 3L)
+  nanotest(is.character(b) || is.character(b$data))
+  Sys.sleep(2.2)
+  nanotestp(m)
+  nanotestp(b)
+  `lang obj` <- quote(m + n + 2L)
+  args <- list(m = 2L, n = 4L)
+  m <- mirai(.expr = `lang obj`, .args = args, .timeout = 2000L)
+  nanotest(call_mirai(m)$data == 8L || is_error_value(m$data))
+  Sys.sleep(2.2)
+  nanotest(daemons(url = value <- mirai:::append_token(sprintf(mirai:::.urlfmt, ""), TRUE), dispatcher = FALSE) == value)
   nanotestz(daemons(0L))
   Sys.sleep(0.5)
   nanotest(daemons(n = 2L, url = value <- "ws://:0", dispatcher = FALSE) != value)
@@ -53,17 +72,21 @@ if (Sys.getenv("NOT_CRAN") == "true") {
   nanotesto(daemons(url = "tcp://:0"))
   nanotestz(daemons(0L))
   Sys.sleep(1L)
-  nanotesto(daemons(1L, dispatcher = FALSE, walltime = 900L, timerstart = 1L, .compute = "new"))
+  nanotesto(daemons(1L, dispatcher = FALSE, walltime = 2000L, timerstart = 1L, cleanup = 0L, .compute = "new"))
   mn <- mirai("test1", .compute = "new")
   mp <- mirai("test2", .compute = "new")
-  nanotest(call_mirai(mn)$data == "test1" || is_error_value(mn$data))
-  nanotest(call_mirai(mp)$data == "test2" || is_error_value(mp$data))
-  nanotesto(daemons(.compute = "new")[["connections"]])
-  Sys.sleep(2.2)
-  nanotestz(daemons(.compute = "new")[["connections"]])
+  Sys.sleep(1L)
+  nanotest(mn$data == "test1" || unresolved(mn$data))
+  nanotest(mp$data == "test2" || unresolved(mp$data))
+  nanotest(daemons(.compute = "new")[["connections"]] %in% c(0L, 1L))
+  Sys.sleep(3L)
+  nanotest(daemons(.compute = "new")[["connections"]] %in% c(0L, 1L))
   nanotestz(daemons(0L, .compute = "new"))
+  Sys.sleep(1L)
+
+  Sys.getenv("NOT_CRAN") == "true" || break
   nanotest(daemons(n = 2, "ws://:0") == 2L)
-  Sys.sleep(1.5)
+  Sys.sleep(1L)
   nanotest(is.matrix(status <- daemons()[["daemons"]]))
   nanotest(is.integer(attr(dn1 <- dimnames(status)[[1L]], "dispatcher_pid")))
   nanotest(is.character(parse1 <- nanonext::parse_url(dn1[1L])))
@@ -72,46 +95,42 @@ if (Sys.getenv("NOT_CRAN") == "true") {
   nanotest(as.integer(parse2[["port"]]) == port)
   nanotest(parse1[["path"]] == "/1")
   nanotest(parse2[["path"]] == "/2")
-  nanotestz(sum(status[, "status_online"]))
-  nanotestz(sum(status[, "status_busy"]))
-  nanotestz(sum(status[, "tasks_assigned"]))
-  nanotestz(sum(status[, "tasks_complete"]))
-  nanotestz(sum(status[, "instance #"]))
+  nanotestz(sum(status[, "online"]))
+  nanotestz(sum(status[, "instance"]))
+  nanotestz(sum(status[, "assigned"]))
+  nanotestz(sum(status[, "complete"]))
+  nanotest(is.character(saisei(i = 1L)))
+  nanotestn(saisei(i = 10L))
+  Sys.sleep(0.5)
   nanotestz(daemons(0))
   Sys.sleep(1L)
-  nanotest(daemons(n = 2, "tcp://:0") == 2L)
-  Sys.sleep(1.5)
+  nanotest(daemons(n = 2, "tcp://127.0.0.1:0") == 2L)
+  Sys.sleep(1L)
+  nanotest(is.integer(launch_server(row.names(daemons()[["daemons"]])[1L], asyncdial = FALSE)))
+  Sys.sleep(1L)
   nanotest(is.matrix(status <- daemons()[["daemons"]]))
   nanotest(is.integer(attr(dn1 <- dimnames(status)[[1L]], "dispatcher_pid")))
   nanotest(is.character(parse1 <- nanonext::parse_url(dn1[1L])))
   nanotest(is.character(parse2 <- nanonext::parse_url(dn1[2L])))
   nanotest((port <- as.integer(parse1[["port"]])) > 0L)
   nanotest(as.integer(parse2[["port"]]) != port)
-  nanotestz(sum(status[, "status_online"]))
-  nanotestz(sum(status[, "status_busy"]))
-  nanotestz(sum(status[, "tasks_assigned"]))
-  nanotestz(sum(status[, "tasks_complete"]))
-  nanotestz(sum(status[, "instance #"]))
+  nanotesto(sum(status[, "online"]))
+  nanotesto(sum(status[, "instance"]))
+  nanotestz(sum(status[, "assigned"]))
+  nanotestz(sum(status[, "complete"]))
   nanotestz(daemons(0))
   Sys.sleep(1L)
-  cleanup <- FALSE
-  nanotesto(daemons(1, dispatcher = TRUE, maxtasks = 10L, cleanup = cleanup))
-  mq <- mirai("server", .timeout = 1000)
+  option <- 15L
+  nanotesto(daemons(1, dispatcher = TRUE, maxtasks = 10L, token = TRUE, lock = TRUE, cleanup = option))
+  mq <- mirai("server", .timeout = 2000)
   nanotest(call_mirai(mq)$data == "server" || is_error_value(mq$data))
   nanotest(is.matrix(status <- daemons()[["daemons"]]))
-  nanotesto(status[, "status_online"])
-  nanotesto(status[, "instance #"])
+  nanotesto(status[, "online"])
+  nanotesto(status[, "instance"])
   nanotestz(daemons(0))
   Sys.sleep(1L)
-}
-Sys.sleep(1L)
-nanotesterr(daemons(url = "URL"), "argument")
-nanotesterr(daemons(-1), "zero or greater")
-nanotesterr(daemons(raw(0L)), "must be numeric")
-nanotesterr(server("URL"), "argument")
-nanotestz(daemons(0L))
-nanotest(is_mirai_interrupt(r <- mirai:::mk_interrupt_error()))
-nanotestp(r)
-nanotestp(m)
-nanotestp(b)
+  break
 
+}
+
+Sys.sleep(1L)
